Package: PureCN
Type: Package
Title: Copy number calling and SNV classification using targeted short
        read sequencing
Version: 2.0.1
Date: 2021-10-29
Authors@R: c(person("Markus", "Riester",
                    role = c("aut", "cre"),
                    email = "markus.riester@novartis.com",
                    comment = c(ORCID = "0000-0002-4759-8332")),
             person("Angad P.", "Singh", role = "aut"))
Description: This package estimates tumor purity, copy number, and loss of
    heterozygosity (LOH), and classifies single nucleotide variants (SNVs) by
    somatic status and clonality. PureCN is designed for targeted short read
    sequencing data, integrates well with standard somatic variant detection
    and copy number pipelines, and has support for tumor samples without
    matching normal samples.
Depends: R (>= 3.5.0), DNAcopy, VariantAnnotation (>= 1.14.1)
Imports: GenomicRanges (>= 1.20.3), IRanges (>= 2.2.1), RColorBrewer,
        S4Vectors, data.table, grDevices, graphics, stats, utils,
        SummarizedExperiment, GenomeInfoDb, GenomicFeatures, Rsamtools,
        Biobase, Biostrings, BiocGenerics, rtracklayer, ggplot2,
        gridExtra, futile.logger, VGAM, tools, methods, mclust, rhdf5,
        Matrix
Suggests: BiocParallel, BiocStyle, PSCBS, R.utils,
        TxDb.Hsapiens.UCSC.hg19.knownGene, copynumber, covr, knitr,
        optparse, org.Hs.eg.db, jsonlite, markdown, rmarkdown, testthat
Enhances: genomicsdb (>= 0.0.3)
VignetteBuilder: knitr
License: Artistic-2.0
URL: https://github.com/lima1/PureCN
biocViews: CopyNumberVariation, Software, Sequencing,
        VariantAnnotation, VariantDetection, Coverage, ImmunoOncology
NeedsCompilation: no
ByteCompile: yes
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/PureCN
git_branch: RELEASE_3_14
git_last_commit: 4e75402
git_last_commit_date: 2021-10-29
Date/Publication: 2021-10-31
Packaged: 2021-10-31 22:21:11 UTC; biocbuild
Author: Markus Riester [aut, cre] (<https://orcid.org/0000-0002-4759-8332>),
  Angad P. Singh [aut]
Maintainer: Markus Riester <markus.riester@novartis.com>
