\name{PureCN-defunct}
\alias{PureCN-defunct}
\title{Defunct functions in package \sQuote{PureCN}}

\description{
  These functions are defunct and no longer available.
}

\details{
  The following functions are defunct; use
  the replacement indicated below:
  \itemize{
    \item{autoCurateResults: no replacement}
    \item{calculateGCContentByInterval: \code{\link{preprocessIntervals}}}
    \item{calculateIntervalWeights: \code{\link{createNormalDatabase}}}
    \item{createExonWeightFile: \code{\link{createNormalDatabase}}}
    \item{createSNPBlacklist: \code{\link{setMappingBiasVcf}}}
    \item{createTargetWeights: \code{\link{createNormalDatabase}}}
    \item{filterTargets: \code{\link{filterIntervals}}}
    \item{findBestNormal: \code{\link{calculateTangentNormal}}}
    \item{getDiploid: no replacement}
    \item{plotBestNormal: no replacement}
    \item{readCoverageGatk: \code{\link{readCoverageFile}}}
  }
}
